/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend

import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.network.ApiCallback
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.partner.friend.model.PickerFriendFilter
import com.kakao.sdk.partner.friend.model.PickerFriendOrder
import com.kakao.sdk.partner.friend.model.PickerFriendType
import com.kakao.sdk.partner.friend.network.FriendApi
import com.kakao.sdk.partner.friend.network.UserScopeApi
import com.kakao.sdk.partner.friend.network.model.*

/**
 * @suppress
 */
internal object Repository {
    private val friendApi: FriendApi = ApiFactory.kapiWithOAuth.create(FriendApi::class.java)
    private val scopesApi: UserScopeApi = ApiFactory.kapi.create(UserScopeApi::class.java)

    var friends = mutableListOf<PickerFriend>()
    var me: Me? = null

    // char picker
    var chats = mutableListOf<PickerChat>()
    var chatMemebers = mutableListOf<PickerFriend>()

    var invitableProfilePermission = false
    private var apiCalled = false

    fun loadFriendsData(
        friendOrder: PickerFriendOrder?,
        friendType: PickerFriendType,
        friendFilter: PickerFriendFilter,
        countryCodes: List<String>?,
        callback: (error: Throwable?) -> Unit
    ) {
        if (apiCalled) {
            return
        }
        apiCalled = true
        pickerFriendsForPartner(
            friendOrder,
            friendType,
            friendFilter,
            countryCodes
        ) { data, error ->
            if (error != null) {
                callback(error)
            } else if (data != null) {
                invitableProfilePermission = data.displayAllProfile
                data.friends.elements?.let {
                    friends.addAll(it)
                    me = data.me
                    callback(null)
                }
            }
        }
    }

    fun loadChatsData(
        filters: List<PickerChatFilter>? = null,
        callback: (error: Throwable?) -> Unit
    ) {
        pickerChatsForPartner(filters) { data, error ->
            if (error != null) {
                callback(error)
            } else if (data != null) {
                data.elements?.let {
                    chats.addAll(it)
                    callback(null)
                }
            }
        }

    }

    fun loadChatMembersData(
        chatId: Long,
        friendsOnly: Boolean? = null,
        includeProfile: Boolean? = null,
        callback: (error: Throwable?) -> Unit
    ) {
        pickerChatMembersForPartner(chatId, friendsOnly, includeProfile) { data, error ->
            if (error != null) {
                callback(error)
            } else if (data != null) {
                data.members?.let {
                    it.forEach { member ->
                        chatMemebers.add(
                            PickerFriend(
                                id = member.id,
                                uuid = member.uuid,
                                profileNickname = member.nickname,
                                profileThumbnailImage = member.thumbnailImageUrl,
                                appRegistered = member.appRegistered,
                                registeredToApp = member.appRegistered,
                                ageFiltered = null,
                                favorite = null,
                                allowedMsg = null,
                                relation = null,
                                talkOs = null,
                                serviceUserId = null,
                                msgAllowed = null
                            )
                        )

                    }
                    callback(null)
                }
            }
        }
    }

    fun getUserScopes(
        scopeGroup: PickerScopeGroup,
        targetIds: List<Long>,
        callback: (userScopes: List<PickerUserScope>?, error: Throwable?) -> Unit
    ) {
        scopesApi.userScopes(scopeGroup, KakaoJson.toJson(targetIds))
            .enqueue(object : ApiCallback<List<PickerUserScope>>() {
                override fun onComplete(model: List<PickerUserScope>?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    private fun pickerFriendsForPartner(
        friendOrder: PickerFriendOrder?,
        friendType: PickerFriendType,
        friendFilter: PickerFriendFilter,
        countryCodes: List<String>?,
        callback: (users: PickerUsers<PickerFriend>?, error: Throwable?) -> Unit
    ) {
        friendApi.friends(
            friendOrder = friendOrder,
            friendType = friendType,
            friendFilter = friendFilter,
            limit = Constants.PARTNER_FRIENDS_API_LIMIT,
            countryCodes = countryCodes?.joinToString(",")
        ).enqueue(object : ApiCallback<PickerUsers<PickerFriend>>() {
            override fun onComplete(model: PickerUsers<PickerFriend>?, error: Throwable?) {
                callback(model, error)
            }
        })
    }

    private fun pickerChatsForPartner(
        filters: List<PickerChatFilter>? = null,
        callback: (users: PickerChats?, error: Throwable?) -> Unit
    ) {
        friendApi.chatList(
            filter = filters?.joinToString(","),
            limit = Constants.PARTNER_CHATS_API_LIMIT
        ).enqueue(object : ApiCallback<PickerChats>() {
            override fun onComplete(model: PickerChats?, error: Throwable?) {
                callback(model, error)
            }
        })
    }

    private fun pickerChatMembersForPartner(
        chatId: Long,
        friendsOnly: Boolean?,
        includeProfile: Boolean?,
        callback: (users: PickerChatMembers?, error: Throwable?) -> Unit
    ) {
        friendApi.chatMembers(
            chatId = chatId,
            friendsOnly = friendsOnly,
            includeProfile = includeProfile
        ).enqueue(object : ApiCallback<PickerChatMembers>() {
            override fun onComplete(model: PickerChatMembers?, error: Throwable?) {
                callback(model, error)
            }
        })
    }

    fun clearData() {
        apiCalled = false
        me = null
        friends.clear()
        chats.clear()
        chatMemebers.clear()
    }
}