/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.sample

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.BaseTransientBottomBar
import com.google.android.material.snackbar.Snackbar
import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.auth.network.RxAuthOperations
import com.kakao.sdk.auth.rx
import com.kakao.sdk.common.model.ApiError
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.partner.auth.model.AgeAuthLevel
import com.kakao.sdk.partner.auth.verifyAge
import com.kakao.sdk.partner.friend.client.PickerClient
import com.kakao.sdk.partner.friend.client.rx
import com.kakao.sdk.partner.friend.model.PickerFriendFilter
import com.kakao.sdk.partner.friend.model.PickerFriendType
import com.kakao.sdk.partner.friend.model.PickerFriendRequestParams
import com.kakao.sdk.partner.friend.model.ViewAppearance
import com.kakao.sdk.partner.talk.*
import com.kakao.sdk.partner.talk.model.Chat
import com.kakao.sdk.partner.talk.model.PartnerFriend
import com.kakao.sdk.partner.user.*
import com.kakao.sdk.partner.user.model.AgeAuthInfo
import com.kakao.sdk.sample.common.OpenFragmentRx
import com.kakao.sdk.sample.common.internal.ApiAdapter
import com.kakao.sdk.sample.common.internal.FriendsActivity
import com.kakao.sdk.sample.common.internal.Log
import com.kakao.sdk.sample.common.internal.PickerItem
import com.kakao.sdk.talk.TalkApiClient
import com.kakao.sdk.talk.rx
import com.kakao.sdk.user.UserApiClient
import com.kakao.sdk.user.rx
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.functions.Action
import io.reactivex.functions.Consumer
import io.reactivex.rxkotlin.addTo
import io.reactivex.schedulers.Schedulers
import kotlinx.android.synthetic.main.fragment_partner.*

/**
 * A simple [Fragment] subclass.
 */
class PartnerFragment : Fragment() {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let { templateIds = it }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_partner, container, false)
    }

    val onSuccess = Consumer<Any> {
        showSnackbar(it)
    }

    val onError = Consumer<Throwable> {
        showSnackbar(it)
    }

    private fun showSnackbar(any: Any) {
        val snackbar = Snackbar.make(view!!, any.toString(), BaseTransientBottomBar.LENGTH_LONG)
        val layout = snackbar.view as Snackbar.SnackbarLayout
        layout.minimumHeight = 100
        snackbar.show()
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val TAG = "KakaoSDKSample"

        viewManager = LinearLayoutManager(context!!)
        viewAdapter = ApiAdapter(
            listOf(
                ApiAdapter.Item.Header("Auth API"),
                ApiAdapter.Item.ApiItem("verifyAge(default) ") {
                    AuthApiClient.rx.verifyAge(context!!)
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")

                        }, { error ->
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("verifyAge(1차 인증) ") {
                    AuthApiClient.rx.verifyAge(context!!, authLevel = AgeAuthLevel.AUTH_LEVEL1)
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }, { error ->
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("verifyAge(2차 인증) ") {
                    AuthApiClient.rx.verifyAge(context!!, authLevel = AgeAuthLevel.AUTH_LEVEL2)
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({
                            Log.i(TAG, "연령 인증 성공")
                            showSnackbar("연령 인증 성공")
                        }, { error ->
                            Log.e(TAG, "연령 인증 실패", error)
                            showSnackbar("연령 인증 실패")
                        }).addTo(disposables)
                },
                ApiAdapter.Item.Header("User API"),
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - empty") {
                    UserApiClient.rx.loginWithKakaoAccount(context!!, mapOf())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ token ->
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }, { error ->
                            Log.e(TAG, "로그인 실패", error)
                        })
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - login '@kakaocorp.com'") {
                    UserApiClient.rx.loginWithKakaoAccount(context!!, mapOf("view" to "login", "email" to "@kakaocorp.com"))
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ token ->
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }, { error ->
                            Log.e(TAG, "로그인 실패", error)
                        })
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("loginWithKakaoAccount(accountParameters:) - signup") {
                    UserApiClient.rx.loginWithKakaoAccount(context!!, mapOf("view" to "signup"))
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ token ->
                            Log.i(TAG, "로그인 성공 ${token.accessToken}")
                        }, { error ->
                            Log.e(TAG, "로그인 실패", error)
                        })
                },
                ApiAdapter.Item.ApiItem("meForPartner()") {
                    UserApiClient.rx.meForPartner()
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("signupForPartner()") {
                    UserApiClient.rx.signupForPartner()
                        .subscribeOn(Schedulers.io())
                        .subscribe(Action { showSnackbar("signupForPartner() 성공")}, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo()") {
                    UserApiClient.rx.ageAuthInfo()
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("ageAuthInfo(account_ci)") {
                    UserApiClient.rx.ageAuthInfo(propertyKeys = listOf("account_ci"))
                        .flatMap { info: AgeAuthInfo? ->
                            if (info?.ciNeedsAgreement == true) {
                                Log.d(TAG, "사용자에게 ci 제공 동의를 받아야 합니다.")

                                // InsufficientScope 에러 생성
                                Single.error(ApiError.fromScopes(listOf("account_ci")))
                            }
                            else {
                                Single.just(info)
                            }
                        }
                        .retryWhen(
                            // InsufficientScope 에러에 대해 추가 동의 후 재요청
                            RxAuthOperations.instance.incrementalAuthorizationRequired(context!!)
                        )
                        .subscribeOn(Schedulers.io())
                        .observeOn(AndroidSchedulers.mainThread())
                        .subscribe({ info ->
                            if (info?.ci != null){
                                Log.i(TAG, "ci 획득  ${info?.ci}")
                                showSnackbar("ci 획득  ${info?.ci}")
                            }
                            else {
                                Log.e(TAG, "사용자 계정에 ci 없음. 꼭 필요하다면 동의항목 설정에서 수집 기능을 활성화 해보세요.")
                            }
                        },{ error ->
                                Log.e(TAG, "ci 조회 실패", error)
                        } ).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("upgradeScopes()") {
                    val scopes = listOf("birthday")
                    UserApiClient.rx.upgradeScopes(scopes)
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.Header("Friend Picker API"),
                ApiAdapter.Item.ApiItem("show friend picker") {
                    PickerClient.rx.selectFriend(
                        context!!,
                        PickerFriendRequestParams(
                            "친구 초대하기",
                            pickerFriendType = PickerFriendType.STORY,
                            pickerFriendFilter = PickerFriendFilter.NONE,
                            showMyProfile = false,
                            viewAppearance = ViewAppearance.LIGHT
                        )
                    ).subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.Header("Talk API"),
                ApiAdapter.Item.ApiItem("friendsForPartner()") {
                    TalkApiClient.rx.friendsForPartner()
                        .retryWhen(RxAuthOperations.instance.incrementalAuthorizationRequired(context!!))
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("chatList()") {
                    TalkApiClient.rx.chatList()
                        .retryWhen(RxAuthOperations.instance.incrementalAuthorizationRequired(context!!))
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("chatMembers()") {
                    TalkApiClient.rx.chatList().flatMap { friends ->
                        friends.elements?.let {
                            if (it.isNotEmpty()) {
                                TalkApiClient.rx.chatMembers(it[0].id)
                            } else {
                                Single.error<KakaoSdkError>(
                                    ClientError(
                                        ClientErrorCause.IllegalState,
                                        "No chats"
                                    )
                                )
                            }
                        }
                    }.retryWhen(RxAuthOperations.instance.incrementalAuthorizationRequired(context!!))
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageForPartner()") {
                    TalkApiClient.rx.friendsForPartner()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    context!!,
                                    it.map(PickerItem.Companion::fromPartnerFriend)
                                )
                            }
                        }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendCustomMessageForPartner(
                                it,
                                templateIds["customMessage"] as Long
                            )
                        }
                        .retryWhen(RxAuthOperations.instance.incrementalAuthorizationRequired(context!!))
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageForPartner()") {
                    TalkApiClient.rx.friendsForPartner()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    context!!,
                                    it.map(PickerItem.Companion::fromPartnerFriend)
                                )
                            }
                        }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendScrapMessageForPartner(
                                it,
                                "https://developers.kakao.com"
                            )
                        }
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageForPartner()") {
                    TalkApiClient.rx.friendsForPartner().flatMap { friends ->
                        friends.elements?.let {
                            FriendsActivity.startForResult(
                                context!!,
                                it.map(PickerItem.Companion::fromPartnerFriend)
                            )
                        }
                    }.flatMap {
                        TalkApiClient.rx.sendDefaultMessageForPartner(
                            it,
                            OpenFragmentRx.defaultLocation
                        )
                    }.retryWhen(RxAuthOperations.instance.incrementalAuthorizationRequired(context!!))
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendCustomMessageToChats()") {
                    TalkApiClient.rx.chatList()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    context!!,
                                    it.map(PickerItem.Companion::fromChat)
                                )
                            }
                        }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendCustomMessageToChats(
                                it.map { it.toLong() },
                                templateIds["customMessage"] as Long
                            )
                        }
                        .retryWhen(RxAuthOperations.instance.incrementalAuthorizationRequired(context!!))
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendDefaultMessageToChats()") {
                    TalkApiClient.rx.chatList()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    context!!,
                                    it.map(PickerItem.Companion::fromChat)
                                )
                            }
                        }.map { it.map { it.toLong() } }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendDefaultMessageToChats(
                                it,
                                OpenFragmentRx.defaultFeed
                            )
                        }
                        .subscribeOn(Schedulers.io())
                        .subscribe(onSuccess, onError).addTo(disposables)
                },
                ApiAdapter.Item.ApiItem("sendScrapMessageToChats()") {
                    TalkApiClient.rx.chatList()
                        .flatMap { friends ->
                            friends.elements?.let {
                                FriendsActivity.startForResult(
                                    context!!,
                                    it.map(PickerItem.Companion::fromChat)
                                )
                            }
                        }
                        .map { it.map { it.toLong() } }
                        .observeOn(Schedulers.io())
                        .flatMap {
                            TalkApiClient.rx.sendScrapMessageToChats(
                                it,
                                "https://developers.kakao.com"
                            )

                        }
                        .subscribeOn(Schedulers.io()).subscribe(onSuccess, onError)
                        .addTo(disposables)
                })
        )

        recyclerView.apply {
            setHasFixedSize(true)
            addItemDecoration(
                DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
            )
            layoutManager = viewManager
            adapter = viewAdapter
        }
    }

    private lateinit var viewAdapter: RecyclerView.Adapter<*>
    private lateinit var viewManager: RecyclerView.LayoutManager
    val disposables = CompositeDisposable()

    private lateinit var templateIds: Bundle


    companion object {
        fun newInstance(templateIds: Bundle): PartnerFragment =
            PartnerFragment().apply {
                arguments = templateIds
            }
    }
}

fun PickerItem.Companion.fromPartnerFriend(friend: PartnerFriend) =
    PickerItem(friend.uuid, friend.profileNickname ?: "", friend.profileThumbnailImage)

fun PickerItem.Companion.fromChat(chat: Chat) =
    PickerItem(
        chat.id.toString(),
        chat.title ?: "",
        if ((chat.displayMemberImages?: emptyList()).isNotEmpty()) chat.displayMemberImages!![0] else chat.imageUrl
    )